"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateRequest = exports.setApiConf = exports.indexRequest = exports.getRequest = exports.deleteRequest = exports.createRequest = void 0;

var _utils = require("./utils.cjs");

let apiConf = {
  baseUrl: "http://localhost"
};

const setApiConf = conf => {
  apiConf = conf;
};

exports.setApiConf = setApiConf;

const indexRequest = async (resourceType, queryParams = {}) => {
  const url = (0, _utils.getUrl)(apiConf.baseUrl || "", `/${resourceType}`);
  const headers = new Headers();
  headers.append("Content-Type", "application/vnd.api+json");
  const conf = {
    method: "GET",
    headers
  };
  Object.keys(queryParams).forEach(key => url.searchParams.append(key, queryParams[key]));
  const response = await fetch(url.href, conf);

  if (!response.ok) {
    throw new Error(`Api call failed with status: ${response.status} and message: ${response.statusText}`);
  }

  return response.json();
};

exports.indexRequest = indexRequest;

const getRequest = async (resourceType, id, queryParams = {}) => {
  const url = (0, _utils.getUrl)(apiConf.baseUrl || "", `/${resourceType}/${id}`);
  const headers = new Headers();
  headers.append("Content-Type", "application/vnd.api+json");
  const conf = {
    method: "GET",
    headers
  };
  Object.keys(queryParams).forEach(key => url.searchParams.append(key, queryParams[key]));
  const response = await fetch(url.href, conf);

  if (!response.ok) {
    throw new Error(`Api call failed with status: ${response.status} and message: ${response.statusText}`);
  }

  return response.json();
};

exports.getRequest = getRequest;

const createRequest = async (resourceType, body) => {
  const url = (0, _utils.getUrl)(apiConf.baseUrl || "", `/${resourceType}`);
  const headers = new Headers();
  headers.append("Content-Type", "application/vnd.api+json");
  const conf = {
    method: "POST",
    headers,
    body: JSON.stringify(body)
  };
  const response = await fetch(url.href, conf);

  if (!response.ok) {
    throw new Error(`Api call failed with status: ${response.status} and message: ${response.statusText}`);
  }

  if (response.status === 204) return "";
  return response.json();
};

exports.createRequest = createRequest;

const updateRequest = async (resourceType, id, body) => {
  const url = (0, _utils.getUrl)(apiConf.baseUrl || "", `/${resourceType}/${id}`);
  const headers = new Headers();
  headers.append("Content-Type", "application/vnd.api+json");
  const conf = {
    method: "PATCH",
    headers,
    body: JSON.stringify(body)
  };
  const response = await fetch(url.href, conf);

  if (!response.ok) {
    throw new Error(`Api call failed with status: ${response.status} and message: ${response.statusText}`);
  }

  if (response.status === 204) return "";
  return response.json();
};

exports.updateRequest = updateRequest;

const deleteRequest = async (resourceType, id) => {
  const url = (0, _utils.getUrl)(apiConf.baseUrl || "", `/${resourceType}/${id}`);
  const headers = new Headers();
  headers.append("Content-Type", "application/vnd.api+json");
  const conf = {
    method: "DELETE",
    headers
  };
  const response = await fetch(url.href, conf);

  if (!response.ok) {
    throw new Error(`Api call failed with status: ${response.status} and message: ${response.statusText}`);
  }

  return response.json();
};

exports.deleteRequest = deleteRequest;