"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.storeVueQuery = void 0;

var _vue = require("vue");

var _vueQuery = require("vue-query");

var _defu = require("defu");

var _requests = require("./requests.cjs");

var _utils = require("./utils.cjs");

const getNullKeys = element => Object.entries(element ?? {}).filter(entry => entry[1] === null).map(entry => entry[0]);

const removeNullKeys = (receivedObject, currentObject) => {
  const nullAttributeKeys = getNullKeys(receivedObject?.attributes);
  const nullRelationshipKeys = getNullKeys(receivedObject?.relationships);
  if (!currentObject) return currentObject;
  const attributes = !!currentObject?.attributes ? {} : { ...currentObject.attributes
  };
  const relationships = !!currentObject?.relationships ? {} : { ...currentObject.relationships
  };
  nullAttributeKeys.forEach(key => delete attributes[key]);
  nullRelationshipKeys.forEach(key => delete relationships[key]);
  return { ...currentObject,
    attributes,
    relationships
  };
};

const storeVueQuery = (store, apiResourceType, queryOptions) => {
  const resourceType = apiResourceType || store.$id;
  const indexKey = (0, _vue.reactive)([resourceType, {}]);
  const indexQuery = (0, _vueQuery.useQuery)(indexKey, () => (0, _requests.indexRequest)(resourceType, indexKey[1]), {
    keepPreviousData: true,
    staleTime: 1e4,
    ...queryOptions?.index
  });

  const index = async (queryParams = {}) => {
    indexKey[1] = queryParams;
    const data = indexQuery.data.value;

    if (data) {
      (0, _utils.processIndexData)(data, store);
    }
  };

  (0, _vue.watch)(indexQuery.data, newData => {
    const data = newData;

    if (data) {
      (0, _utils.processIndexData)(data, store);
    }
  });
  const getKey = (0, _vue.reactive)([resourceType, "", {}]);
  const getQuery = (0, _vueQuery.useQuery)(getKey, () => (0, _requests.getRequest)(resourceType, getKey[1], getKey[2]), {
    staleTime: 1e4,
    ...queryOptions?.get
  });

  const get = async (id, queryParams = {}) => {
    getKey[1] = id;
    getKey[2] = queryParams;
    const data = getQuery.data.value;

    if (data) {
      (0, _utils.processGetData)(data, store);
    }
  };

  (0, _vue.watch)(getQuery.data, newData => {
    const data = newData;

    if (data) {
      (0, _utils.processGetData)(data, store);
    }
  });
  const createQuery = (0, _vueQuery.useMutation)(body => (0, _requests.createRequest)(resourceType, body), {
    onSuccess: (json, body) => {
      if (typeof json === "string") {
        const currentData = store.data[body.id || ""];
        const transformedData = removeNullKeys(body, currentData);
        store.data[body.id || ""] = (0, _defu.defu)(body, transformedData);
      } else {
        const elementData = json.data;
        const transformedData = removeNullKeys(elementData, store.data[elementData.id]);
        store.data[elementData.id] = (0, _defu.defu)(elementData, transformedData);
      }
    },
    ...queryOptions?.create
  });

  const create = async body => {
    createQuery.mutate(body);
  };

  const updateQuery = (0, _vueQuery.useMutation)(params => (0, _requests.updateRequest)(resourceType, params.id, params.body), {
    onSuccess: (json, variables) => {
      if (typeof json === "string") {
        store.get(variables.id);
      } else {
        const elementData = json.data;
        const transformedData = removeNullKeys(elementData, store.data[elementData.id]);
        store.data[elementData.id] = (0, _defu.defu)(elementData, transformedData);
      }
    },
    ...queryOptions?.update
  });

  const update = async (id, body) => {
    updateQuery.mutate({
      id,
      body
    });
  };

  const deleteQuery = (0, _vueQuery.useMutation)(id => (0, _requests.deleteRequest)(resourceType, id), {
    onSuccess: (_, id) => {
      delete store.data[id];
    },
    ...queryOptions?.delete
  });

  const vDelete = async id => {
    deleteQuery.mutate(id);
  };

  return {
    index,
    indexQuery,
    get,
    getQuery,
    create,
    createQuery,
    update,
    updateQuery,
    delete: vDelete,
    deleteQuery
  };
};

exports.storeVueQuery = storeVueQuery;